@@ -96,7 +96,8 @@ static void janus_dtls_notify_state_change(janus_dtls_srtp *dtls) {
 	json_object_set_new(info, "stream_id", json_integer(stream->stream_id));
 	json_object_set_new(info, "component_id", json_integer(component->component_id));
 	json_object_set_new(info, "retransmissions", json_integer(dtls->retransmissions));
-	janus_events_notify_handlers(JANUS_EVENT_TYPE_WEBRTC, session->session_id, handle->handle_id, handle->opaque_id, info);
+	janus_events_notify_handlers(JANUS_EVENT_TYPE_WEBRTC, JANUS_EVENT_SUBTYPE_WEBRTC_DTLS,
+		session->session_id, handle->handle_id, handle->opaque_id, info);
 }
 
 gboolean janus_is_dtls(char *buf) {
@@ -104,14 +105,25 @@ gboolean janus_is_dtls(char *buf) {
 }
 
 /* DTLS stuff */
-#define DTLS_CIPHERS	"HIGH:!aNULL:!MD5:!RC4"
+#define DTLS_DEFAULT_CIPHERS	"DEFAULT:!NULL:!aNULL:!SHA256:!SHA384:!aECDH:!AESGCM+AES256:!aPSK"
+static const char *dtls_ciphers = DTLS_DEFAULT_CIPHERS;
 /* Duration for the self-generated certs: 1 year */
 #define DTLS_AUTOCERT_DURATION	60*60*24*365
+/* NIST P-256 elliptic curve used for private key generation */
+#define DTLS_ELLIPTIC_CURVE NID_X9_62_prime256v1
+
+/* By default we always accept self-signed certificates (that's what almost
+ * all of the existing WebRTC implementations do today), but if told so we
+ * can be configured to reject them, and validate them instead */
+static gboolean dtls_selfsigned_certs_ok = TRUE;
+gboolean janus_dtls_are_selfsigned_certs_ok(void) {
+	return dtls_selfsigned_certs_ok;
+}
 
 /* DTLS timeout base to enforce: notice that this can currently only be
  * modified if you're using BoringSSL, as OpenSSL uses 1s (1000ms) and
  * that value cannot be modified (it will in OpenSSL v1.1.1) */
-static guint dtls_timeout_base = 1000;
+static guint16 dtls_timeout_base = 1000;
 
 static SSL_CTX *ssl_ctx = NULL;
 static X509 *ssl_cert = NULL;
@@ -173,39 +185,15 @@ static void janus_dtls_cb_openssl_lock(int mode, int type, const char *file, int
 #endif
 
 
-static int janus_dtls_generate_keys(X509 **certificate, EVP_PKEY **private_key) {
+static int janus_dtls_generate_keys(X509 **certificate, EVP_PKEY **private_key, gboolean rsa_private_key) {
 	static const int num_bits = 2048;
 	BIGNUM *bne = NULL;
 	RSA *rsa_key = NULL;
 	X509_NAME *cert_name = NULL;
+	EC_KEY *ecc_key = NULL;
 
 	JANUS_LOG(LOG_VERB, "Generating DTLS key / cert\n");
 
-	/* Create a big number object. */
-	bne = BN_new();
-	if(!bne) {
-		JANUS_LOG(LOG_FATAL, "BN_new() failed\n");
-		goto error;
-	}
-
-	if(!BN_set_word(bne, RSA_F4)) {  /* RSA_F4 == 65537 */
-		JANUS_LOG(LOG_FATAL, "BN_set_word() failed\n");
-		goto error;
-	}
-
-	/* Generate a RSA key. */
-	rsa_key = RSA_new();
-	if(!rsa_key) {
-		JANUS_LOG(LOG_FATAL, "RSA_new() failed\n");
-		goto error;
-	}
-
-	/* This takes some time. */
-	if(!RSA_generate_key_ex(rsa_key, num_bits, bne, NULL)) {
-		JANUS_LOG(LOG_FATAL, "RSA_generate_key_ex() failed\n");
-		goto error;
-	}
-
 	/* Create a private key object (needed to hold the RSA key). */
 	*private_key = EVP_PKEY_new();
 	if(!*private_key) {
@@ -213,12 +201,63 @@ static int janus_dtls_generate_keys(X509 **certificate, EVP_PKEY **private_key)
 		goto error;
 	}
 
-	if(!EVP_PKEY_assign_RSA(*private_key, rsa_key)) {
-		JANUS_LOG(LOG_FATAL, "EVP_PKEY_assign_RSA() failed\n");
-		goto error;
+
+	if(rsa_private_key) {
+		/* Create a big number object. */
+		bne = BN_new();
+		if(!bne) {
+			JANUS_LOG(LOG_FATAL, "BN_new() failed\n");
+			goto error;
+		}
+
+		if(!BN_set_word(bne, RSA_F4)) {  /* RSA_F4 == 65537 */
+			JANUS_LOG(LOG_FATAL, "BN_set_word() failed\n");
+			goto error;
+		}
+
+		/* Generate a RSA key. */
+		rsa_key = RSA_new();
+		if(!rsa_key) {
+			JANUS_LOG(LOG_FATAL, "RSA_new() failed\n");
+			goto error;
+		}
+
+		/* This takes some time. */
+		if(!RSA_generate_key_ex(rsa_key, num_bits, bne, NULL)) {
+			JANUS_LOG(LOG_FATAL, "RSA_generate_key_ex() failed\n");
+			goto error;
+		}
+
+		if(!EVP_PKEY_assign_RSA(*private_key, rsa_key)) {
+			JANUS_LOG(LOG_FATAL, "EVP_PKEY_assign_RSA() failed\n");
+			goto error;
+		}
+
+		/* The RSA key now belongs to the private key, so don't clean it up separately. */
+		rsa_key = NULL;
+	} else {
+		/* Create key with curve dictated by DTLS_ELLIPTIC_CURVE */
+		if((ecc_key = EC_KEY_new_by_curve_name(DTLS_ELLIPTIC_CURVE)) == NULL) {
+			JANUS_LOG(LOG_FATAL, "EC_KEY_new_by_curve_name() failed\n");
+			goto error;
+		}
+
+		EC_KEY_set_asn1_flag(ecc_key, OPENSSL_EC_NAMED_CURVE);
+
+		/* This takes some time. */
+		if(EC_KEY_generate_key(ecc_key) == 0) {
+			JANUS_LOG(LOG_FATAL, "EC_KEY_generate_key() failed\n");
+			goto error;
+		}
+
+		if(EVP_PKEY_assign_EC_KEY(*private_key, ecc_key) == 0) {
+			JANUS_LOG(LOG_FATAL, "EVP_PKEY_assign_EC_KEY() failed\n");
+			goto error;
+		}
+
+		/* The EC key now belongs to the private key, so don't clean it up separately. */
+		ecc_key = NULL;
 	}
-	/* The RSA key now belongs to the private key, so don't clean it up separately. */
-	rsa_key = NULL;
 
 	/* Create the X509 certificate. */
 	*certificate = X509_new();
@@ -273,6 +312,8 @@ error:
 		BN_free(bne);
 	if(rsa_key && !*private_key)
 		RSA_free(rsa_key);
+	if(ecc_key && !*private_key)
+		EC_KEY_free(ecc_key);
 	if(*private_key)
 		EVP_PKEY_free(*private_key);  /* This also frees the RSA key. */
 	if(*certificate)
@@ -329,7 +370,8 @@ const char *janus_get_ssl_version(void) {
 }
 
 /* DTLS-SRTP initialization */
-gint janus_dtls_srtp_init(const char *server_pem, const char *server_key, const char *password, guint timeout) {
+gint janus_dtls_srtp_init(const char *server_pem, const char *server_key, const char *password,
+		const char *ciphers, guint16 timeout, gboolean rsa_private_key, gboolean accept_selfsigned) {
 	const char *crypto_lib = NULL;
 #if JANUS_USE_OPENSSL_PRE_1_1_API
 #if defined(LIBRESSL_VERSION_NUMBER)
@@ -380,7 +422,7 @@ gint janus_dtls_srtp_init(const char *server_pem, const char *server_key, const
 
 	if(!server_pem && !server_key) {
 		JANUS_LOG(LOG_WARN, "No cert/key specified, autogenerating some...\n");
-		if(janus_dtls_generate_keys(&ssl_cert, &ssl_key) != 0) {
+		if(janus_dtls_generate_keys(&ssl_cert, &ssl_key, rsa_private_key) != 0) {
 			JANUS_LOG(LOG_FATAL, "Error generating DTLS key/certificate\n");
 			return -2;
 		}
@@ -419,11 +461,16 @@ gint janus_dtls_srtp_init(const char *server_pem, const char *server_key, const
 	}
 	*(lfp-1) = 0;
 	JANUS_LOG(LOG_INFO, "Fingerprint of our certificate: %s\n", local_fingerprint);
-	SSL_CTX_set_cipher_list(ssl_ctx, DTLS_CIPHERS);
+	if(ciphers)
+		dtls_ciphers = ciphers;
+	if(SSL_CTX_set_cipher_list(ssl_ctx, dtls_ciphers) == 0) {
+		JANUS_LOG(LOG_FATAL, "Error setting cipher list (%s)\n", ERR_reason_error_string(ERR_get_error()));
+		return -8;
+	}
 
 	if(janus_dtls_bio_agent_init() < 0) {
 		JANUS_LOG(LOG_FATAL, "Error initializing BIO agent\n");
-		return -8;
+		return -9;
 	}
 
 	dtls_timeout_base = timeout;
@@ -436,8 +483,15 @@ gint janus_dtls_srtp_init(const char *server_pem, const char *server_key, const
 	/* Initialize libsrtp */
 	if(srtp_init() != srtp_err_status_ok) {
 		JANUS_LOG(LOG_FATAL, "Ops, error setting up libsrtp?\n");
-		return 5;
+		return -10;
 	}
+
+	/* Finally, let's set our policy with respect to DTLS self signed certificates */
+	dtls_selfsigned_certs_ok = accept_selfsigned;
+	if(!dtls_selfsigned_certs_ok) {
+		JANUS_LOG(LOG_WARN, "WebRTC PeerConnections with self-signed certificates will NOT be accepted\n");
+	}
+
 	return 0;
 }
 
@@ -551,7 +605,7 @@ janus_dtls_srtp *janus_dtls_srtp_create(void *ice_component, janus_dtls_role rol
 	SSL_set_tmp_ecdh(dtls->ssl, ecdh);
 	EC_KEY_free(ecdh);
 #ifdef HAVE_DTLS_SETTIMEOUT
-	JANUS_LOG(LOG_VERB, "[%"SCNu64"]   Setting DTLS initial timeout: %u\n", handle->handle_id, dtls_timeout_base);
+	JANUS_LOG(LOG_VERB, "[%"SCNu64"]   Setting DTLS initial timeout: %"SCNu16"ms\n", handle->handle_id, dtls_timeout_base);
 	DTLSv1_set_initial_timeout_duration(dtls->ssl, dtls_timeout_base);
 #endif
 	dtls->ready = 0;
@@ -961,14 +1015,26 @@ void janus_dtls_callback(const SSL *ssl, int where, int ret) {
 /* DTLS certificate verification callback */
 int janus_dtls_verify_callback(int preverify_ok, X509_STORE_CTX *ctx) {
 	/* We just use the verify_callback to request a certificate from the client */
-	return 1;
+	int err = X509_STORE_CTX_get_error(ctx);
+	if(err == X509_V_ERR_DEPTH_ZERO_SELF_SIGNED_CERT || err == X509_V_ERR_SELF_SIGNED_CERT_IN_CHAIN) {
+		/* Self signed certificate: by default we always accept it */
+		if(!dtls_selfsigned_certs_ok) {
+			/* ... unless we're enforcing validation */
+			return 0;
+		}
+	}
+	/* We always reject expired certificates, even when self-signed */
+	if(err == X509_V_ERR_CERT_HAS_EXPIRED)
+		return 0;
+	/* Return a success if we're ok with self-signed, the result of the validation otherwise */
+	return dtls_selfsigned_certs_ok ? 1 : (err == X509_V_OK);
 }
 
 #ifdef HAVE_SCTP
-void janus_dtls_wrap_sctp_data(janus_dtls_srtp *dtls, char *label, char *buf, int len) {
+void janus_dtls_wrap_sctp_data(janus_dtls_srtp *dtls, char *label, gboolean textdata, char *buf, int len) {
 	if(dtls == NULL || !dtls->ready || dtls->sctp == NULL || buf == NULL || len < 1)
 		return;
-	janus_sctp_send_data(dtls->sctp, label, buf, len);
+	janus_sctp_send_data(dtls->sctp, label, textdata, buf, len);
 }
 
 int janus_dtls_send_sctp_data(janus_dtls_srtp *dtls, char *buf, int len) {
@@ -982,7 +1048,7 @@ int janus_dtls_send_sctp_data(janus_dtls_srtp *dtls, char *buf, int len) {
 	return res;
 }
 
-void janus_dtls_notify_data(janus_dtls_srtp *dtls, char *label, char *buf, int len) {
+void janus_dtls_notify_data(janus_dtls_srtp *dtls, char *label, gboolean textdata, char *buf, int len) {
 	if(dtls == NULL || buf == NULL || len < 1)
 		return;
 	janus_ice_component *component = (janus_ice_component *)dtls->component;
@@ -1000,7 +1066,7 @@ void janus_dtls_notify_data(janus_dtls_srtp *dtls, char *label, char *buf, int l
 		JANUS_LOG(LOG_ERR, "No handle...\n");
 		return;
 	}
-	janus_ice_incoming_data(handle, label, buf, len);
+	janus_ice_incoming_data(handle, label, textdata, buf, len);
 }
 #endif
 
