@@ -12,8 +12,8 @@
  * \ref protocols
  */
 
-#ifndef _JANUS_DTLS_H
-#define _JANUS_DTLS_H
+#ifndef JANUS_DTLS_H
+#define JANUS_DTLS_H
 
 #include <inttypes.h>
 #include <glib.h>
@@ -32,13 +32,19 @@ const char *janus_get_ssl_version(void);
  * @param[in] server_pem Path to the certificate to use
  * @param[in] server_key Path to the key to use
  * @param[in] password Password needed to use the key, if any
- * @param[in] timeout DTLS timeout base to use for retransmissions (ignored if not using BoringSSL)
+ * @param[in] ciphers DTLS ciphers to use (will use hardcoded defaults, if NULL)
+ * @param[in] timeout DTLS timeout base, in ms, to use for retransmissions (ignored if not using BoringSSL)
+ * @param[in] rsa_private_key Whether RSA certificates should be generated, instead of NIST P-256
+ * @param[in] accept_selfsigned Whether to accept self-signed certificates (default) or enforce validation
  * @returns 0 in case of success, a negative integer on errors */
-gint janus_dtls_srtp_init(const char *server_pem, const char *server_key, const char *password, guint timeout);
+gint janus_dtls_srtp_init(const char *server_pem, const char *server_key, const char *password,
+	const char *ciphers, guint16 timeout, gboolean rsa_private_key, gboolean accept_selfsigned);
 /*! \brief Method to cleanup DTLS stuff before exiting */
 void janus_dtls_srtp_cleanup(void);
 /*! \brief Method to return a string representation (SHA-256) of the certificate fingerprint */
 gchar *janus_dtls_get_local_fingerprint(void);
+/*! \brief Method to check whether DTLS self-signed certificates are ok (default) or not */
+gboolean janus_dtls_are_selfsigned_certs_ok(void);
 
 
 /*! \brief DTLS roles */
@@ -144,9 +150,10 @@ int janus_dtls_verify_callback(int preverify_ok, X509_STORE_CTX *ctx);
 /*! \brief Callback (called from the ICE handle) to encapsulate in DTLS outgoing SCTP data (DataChannel)
  * @param[in] dtls The janus_dtls_srtp instance to use
  * @param[in] label The label of the data channel to use
+ * @param[in] textdata Whether the buffer is text (domstring) or binary data
  * @param[in] buf The data buffer to encapsulate
  * @param[in] len The data length */
-void janus_dtls_wrap_sctp_data(janus_dtls_srtp *dtls, char *label, char *buf, int len);
+void janus_dtls_wrap_sctp_data(janus_dtls_srtp *dtls, char *label, gboolean textdata, char *buf, int len);
 
 /*! \brief Callback (called from the SCTP stack) to encapsulate in DTLS outgoing SCTP data (DataChannel)
  * @param[in] dtls The janus_dtls_srtp instance to use
@@ -158,9 +165,10 @@ int janus_dtls_send_sctp_data(janus_dtls_srtp *dtls, char *buf, int len);
 /*! \brief Callback to be notified about incoming SCTP data (DataChannel) to forward to the handle
  * @param[in] dtls The janus_dtls_srtp instance to use
  * @param[in] label The label of the data channel the message is from
+ * @param[in] textdata Whether the buffer is text (domstring) or binary data
  * @param[in] buf The data buffer
  * @param[in] len The data length */
-void janus_dtls_notify_data(janus_dtls_srtp *dtls, char *label, char *buf, int len);
+void janus_dtls_notify_data(janus_dtls_srtp *dtls, char *label, gboolean textdata, char *buf, int len);
 #endif
 
 /*! \brief DTLS retransmission timer
