@@ -58,7 +58,7 @@ static AVCodecContext *vEncoder;
 static int max_width = 0, max_height = 0, fps = 0;
 
 
-int janus_pp_h264_create(char *destination, char *metadata) {
+int janus_pp_h264_create(char *destination, char *metadata, gboolean faststart) {
 	if(destination == NULL)
 		return -1;
 	/* Setup FFmpeg */
@@ -133,11 +133,16 @@ int janus_pp_h264_create(char *destination, char *metadata) {
 	//~ if (fctx->flags & AVFMT_GLOBALHEADER)
 		vStream->codec->flags |= CODEC_FLAG_GLOBAL_HEADER;
 #endif
-	if(avio_open(&fctx->pb, fctx->filename, AVIO_FLAG_WRITE) < 0) {
-		JANUS_LOG(LOG_ERR, "Error opening file for output\n");
+	AVDictionary *options = NULL;
+	if(faststart)
+		av_dict_set(&options, "movflags", "+faststart", 0);
+
+	int res = avio_open2(&fctx->pb, fctx->filename, AVIO_FLAG_WRITE, NULL, &options);
+	if(res < 0) {
+		JANUS_LOG(LOG_ERR, "Error opening file for output (%d)\n", res);
 		return -1;
 	}
-	if(avformat_write_header(fctx, NULL) < 0) {
+	if(avformat_write_header(fctx, &options) < 0) {
 		JANUS_LOG(LOG_ERR, "Error writing header\n");
 		return -1;
 	}
@@ -353,7 +358,7 @@ int janus_pp_h264_process(FILE *file, janus_pp_frame_packet *list, int *working)
 
 	int bytes = 0, numBytes = max_width*max_height*3;	/* FIXME */
 	uint8_t *received_frame = g_malloc0(numBytes);
-	uint8_t *buffer = g_malloc0(10000), *start = buffer;
+	uint8_t *buffer = g_malloc0(numBytes), *start = buffer;
 	int len = 0, frameLen = 0;
 	int keyFrame = 0;
 	gboolean keyframe_found = FALSE;
@@ -422,6 +427,14 @@ int janus_pp_h264_process(FILE *file, janus_pp_frame_packet *list, int *working)
 				while(tot > 0) {
 					memcpy(&psize, buffer, 2);
 					psize = ntohs(psize);
+					if((frameLen + psize) >= numBytes) {
+						JANUS_LOG(LOG_ERR, "Invalid size %u + %"SCNu16" (exceeds buffer size)\n", frameLen, psize);
+						/* Done, we'll wait for the next video data to write the frame */
+						if(tmp->next == NULL || tmp->next->ts > tmp->ts)
+							break;
+						tmp = tmp->next;
+						continue;
+					}
 					buffer += 2;
 					tot -= 2;
 					/* Now we have a single NAL */
