@@ -22,6 +22,26 @@
 #include "rtcp.h"
 #include "utils.h"
 
+
+/* Transport CC statuses */
+typedef enum janus_rtp_packet_status {
+	janus_rtp_packet_status_notreceived = 0,
+	janus_rtp_packet_status_smalldelta = 1,
+	janus_rtp_packet_status_largeornegativedelta = 2,
+	janus_rtp_packet_status_reserved = 3
+} janus_rtp_packet_status;
+static const char *janus_rtp_packet_status_description(janus_rtp_packet_status status) {
+	switch(status) {
+		case janus_rtp_packet_status_notreceived: return "notreceived";
+		case janus_rtp_packet_status_smalldelta: return "smalldelta";
+		case janus_rtp_packet_status_largeornegativedelta: return "largeornegativedelta";
+		case janus_rtp_packet_status_reserved: return "reserved";
+		default: break;
+	}
+	return NULL;
+}
+
+
 gboolean janus_is_rtcp(char *buf, guint len) {
 	if (len < 8)
 		return FALSE;
@@ -134,7 +154,7 @@ guint32 janus_rtcp_get_receiver_ssrc(char *packet, int len) {
 	return 0;
 }
 
-/* Helper to handle an incoming SR: triggered by a call to janus_rtcp_fix_ssrc with fixssrc=0 */
+/* Helper to handle an incoming SR: triggered by a call to janus_rtcp_fix_ssrc with a valid context pointer */
 static void janus_rtcp_incoming_sr(janus_rtcp_context *ctx, janus_rtcp_sr *sr) {
 	if(ctx == NULL)
 		return;
@@ -146,6 +166,111 @@ static void janus_rtcp_incoming_sr(janus_rtcp_context *ctx, janus_rtcp_sr *sr) {
 	ctx->lsr = (ntp >> 16);
 }
 
+/* Helper to handle an incoming transport-cc feedback: triggered by a call to janus_rtcp_fix_ssrc a valid context pointer */
+static void janus_rtcp_incoming_transport_cc(janus_rtcp_context *ctx, janus_rtcp_fb *twcc, int total) {
+	if(ctx == NULL || twcc == NULL || total < 20)
+		return;
+	if(!janus_rtcp_check_fci((janus_rtcp_header *)twcc, total, 4))
+		return;
+	/* Parse the header first */
+	uint8_t *data = (uint8_t *)twcc->fci;
+	uint16_t base_seq = 0, status_count = 0;
+	uint32_t reference = 0;
+	uint8_t fb_pkt = 0;
+	memcpy(&base_seq, data, sizeof(uint16_t));
+	base_seq = ntohs(base_seq);
+	memcpy(&status_count, data+2, sizeof(uint16_t));
+	status_count = ntohs(status_count);
+	memcpy(&reference, data+4, sizeof(uint32_t));
+	reference = ntohl(reference) >> 8;
+	fb_pkt = *(data+7);
+	JANUS_LOG(LOG_HUGE, "[TWCC] seq=%"SCNu16", psc=%"SCNu16", ref=%"SCNu32", fbpc=%"SCNu8"\n",
+		base_seq, status_count, reference, fb_pkt);
+	/* Now traverse the feedback: packet chunks first, and then recv deltas */
+	total -= 20;
+	data += 8;
+	uint16_t psc = status_count;
+	uint16_t chunk = 0;
+	uint8_t t = 0, ss = 0, s = 0, length = 0;
+	/* Iterate on all packet chunks */
+	JANUS_LOG(LOG_HUGE, "[TWCC] Chunks:\n");
+	uint16_t num = 0;
+	GList *list = NULL;
+	while(psc > 0 && total > 1) {
+		num++;
+		memcpy(&chunk, data, sizeof(uint16_t));
+		chunk = ntohs(chunk);
+		t = (chunk & 0x8000) >> 15;
+		if(t == 0) {
+			/* Run length */
+			s = (chunk & 0x6000) >> 13;
+			length = (chunk & 0x1FFF);
+			JANUS_LOG(LOG_HUGE, "  [%"SCNu16"] t=run-length, s=%s, l=%"SCNu8"\n", num,
+				janus_rtp_packet_status_description(s), length);
+			while(length > 0 && psc > 0) {
+				list = g_list_prepend(list, GUINT_TO_POINTER(s));
+				length--;
+				psc--;
+			}
+		} else {
+			/* Status vector */
+			ss = (chunk & 0x4000) >> 14;
+			length = (s ? 7 : 14);
+			JANUS_LOG(LOG_HUGE, "  [%"SCNu16"] t=status-vector, ss=%s, l=%"SCNu8"\n", num,
+				s ? "2-bit" : "bit", length);
+			while(length > 0 && psc > 0) {
+				if(!ss)
+					s = (chunk & (1 << (length-1))) ? janus_rtp_packet_status_smalldelta : janus_rtp_packet_status_notreceived;
+				else
+					s = (chunk & (3 << (2*length-2))) >> (2*length-2);
+				list = g_list_prepend(list, GUINT_TO_POINTER(s));
+				length--;
+				psc--;
+			}
+		}
+		total -= 2;
+		data += 2;
+	}
+	if(psc > 0) {
+		/* Incomplete feedback? Drop... */
+		g_list_free(list);
+		return;
+	}
+	list = g_list_reverse(list);
+	/* Iterate on all recv deltas */
+	JANUS_LOG(LOG_HUGE, "[TWCC] Recv Deltas (%d/%"SCNu16"):\n", g_list_length(list), status_count);
+	num = 0;
+	uint16_t delta = 0;
+	uint32_t delta_us = 0;
+	GList *iter = list;
+	while(iter != NULL && total > 0) {
+		num++;
+		delta = 0;
+		s = GPOINTER_TO_UINT(iter->data);
+		if(s == janus_rtp_packet_status_smalldelta) {
+			/* Small delta = 1 byte */
+			delta = *data;
+			total--;
+			data++;
+		} else if(s == janus_rtp_packet_status_largeornegativedelta) {
+			/* Large or negative delta = 2 bytes */
+			if(total < 2)
+				break;
+			memcpy(&delta, data, sizeof(uint16_t));
+			delta = ntohs(delta);
+			total -= 2;
+			data += 2;
+		}
+		delta_us = delta*250;
+		/* Print summary */
+		JANUS_LOG(LOG_HUGE, "  [%02"SCNu16"][%"SCNu16"] %s (%"SCNu32"us)\n", num, base_seq+num-1,
+			janus_rtp_packet_status_description(s), delta_us);
+		iter = iter->next;
+	}
+	/* TODO Update the context with the feedback we got */
+	g_list_free(list);
+}
+
 /* Link quality estimate filter coefficient */
 #define LINK_QUALITY_FILTER_K 3.0
 
@@ -438,6 +563,9 @@ int janus_rtcp_fix_ssrc(janus_rtcp_context *ctx, char *packet, int len, int fixs
 						uint32_t *ssrc = (uint32_t *)rtcpfb->fci;
 						*ssrc = htonl(newssrcr);
 					}
+				} else if(fmt == 15) {	/* transport-cc */
+					/* If an RTCP context was provided, parse this transport-cc feedback */
+					janus_rtcp_incoming_transport_cc(ctx, rtcpfb, total);
 				} else {
 					JANUS_LOG(LOG_HUGE, "     #%d ??? -- RTPFB (205, fmt=%d)\n", pno, fmt);
 				}
@@ -626,15 +754,19 @@ char *janus_rtcp_filter(char *packet, int len, int *newlen) {
 }
 
 
-int janus_rtcp_process_incoming_rtp(janus_rtcp_context *ctx, char *packet, int len, gboolean rfc4588_pkt, gboolean rfc4588_enabled, gboolean retransmissions_disabled) {
+int janus_rtcp_process_incoming_rtp(janus_rtcp_context *ctx, char *packet, int len,
+		gboolean rfc4588_pkt, gboolean rfc4588_enabled, gboolean retransmissions_disabled,
+		GHashTable *clock_rates) {
 	if(ctx == NULL || packet == NULL || len < 1)
 		return -1;
 
-	/* First of all, let's check if this is G.711: in case we may need to change the timestamp base */
+	/* First of all, let's check if we need to change the timestamp base */
 	janus_rtp_header *rtp = (janus_rtp_header *)packet;
 	int pt = rtp->type;
-	if((pt == 0 || pt == 8) && (ctx->tb == 48000))
-		ctx->tb = 8000;
+	uint32_t clock_rate = clock_rates ?
+		GPOINTER_TO_UINT(g_hash_table_lookup(clock_rates, GINT_TO_POINTER(pt))) : 0;
+	if(clock_rate > 0 && ctx->tb != clock_rate)
+		ctx->tb = clock_rate;
 	/* Now parse this RTP packet header and update the rtcp_context instance */
 	uint16_t seq_number = ntohs(rtp->seq_number);
 	if(ctx->base_seq == 0 && ctx->seq_cycle == 0)
@@ -1364,13 +1496,6 @@ int janus_rtcp_nacks(char *packet, int len, GSList *nacks) {
 	return words*4+4;
 }
 
-typedef enum janus_rtp_packet_status {
-	janus_rtp_packet_status_notreceived = 0,
-	janus_rtp_packet_status_smalldelta = 1,
-	janus_rtp_packet_status_largeornegativedelta = 2,
-	janus_rtp_packet_status_reserved = 3
-} janus_rtp_packet_status;
-
 int janus_rtcp_transport_wide_cc_feedback(char *packet, size_t size, guint32 ssrc, guint32 media, guint8 feedback_packet_count, GQueue *transport_wide_cc_stats) {
 	if(packet == NULL || size < sizeof(janus_rtcp_header) || transport_wide_cc_stats == NULL || g_queue_is_empty(transport_wide_cc_stats))
 		return -1;
@@ -1397,11 +1522,11 @@ int janus_rtcp_transport_wide_cc_feedback(char *packet, size_t size, guint32 ssr
 	/*
 		0                   1                   2                   3
 		0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
-	       +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
-	       |      base sequence number     |      packet status count      |
-	       +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
-	       |                 reference time                | fb pkt. count |
-	       +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
+		+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
+		|      base sequence number     |      packet status count      |
+		+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
+		|                 reference time                | fb pkt. count |
+		+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 	 */
 
 	/* The packet as unsigned */
@@ -1441,11 +1566,12 @@ int janus_rtcp_transport_wide_cc_feedback(char *packet, size_t size, guint32 ssr
 				/* Got it  */
 				first_received = TRUE;
 				/* Set it */
-				reference_time = (stat->timestamp/64000);
+				reference_time = stat->timestamp / 64000;
 				/* Get initial time */
 				timestamp = reference_time * 64000;
-				/* also in bufffer */
-				janus_set3(data, reference_time_pos, reference_time);
+				/* also in buffer */
+				/* (use only 23 bits of reference_time) */
+				janus_set3(data, reference_time_pos, (reference_time & 0x007FFFFF));
 			}
 
 			/* Get delta */
@@ -1454,7 +1580,7 @@ int janus_rtcp_transport_wide_cc_feedback(char *packet, size_t size, guint32 ssr
 			else
 				delta = -(int)((timestamp-stat->timestamp)/250);
 			/* If it is negative or too big */
-			if (delta<0 || delta> 127) {
+			if (delta<0 || delta> 255) {
 				/* Big one */
 				status = janus_rtp_packet_status_largeornegativedelta;
 			} else {
@@ -1462,12 +1588,13 @@ int janus_rtcp_transport_wide_cc_feedback(char *packet, size_t size, guint32 ssr
 				status = janus_rtp_packet_status_smalldelta;
 			}
 			/* Store delta */
+			/* Overflows are possible here */
 			g_queue_push_tail(deltas, GINT_TO_POINTER(delta));
 			/* Set last time */
 			timestamp = stat->timestamp;
 		}
 
-		/* Check if all previoues ones were equal and this one the firt different */
+		/* Check if all previoues ones were equal and this one the first different */
 		if (all_same && last_status!=janus_rtp_packet_status_reserved && status!=last_status) {
 			/* How big was the same run */
 			if (g_queue_get_length(statuses)>7) {
@@ -1476,9 +1603,9 @@ int janus_rtcp_transport_wide_cc_feedback(char *packet, size_t size, guint32 ssr
 				/*
 					0                   1
 					0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5
-				       +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
-				       |T| S |       Run Length        |
-				       +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
+					+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
+					|T| S |       Run Length        |
+					+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 					T = 0
 				 */
 				word = janus_push_bits(word, 1, 0);
@@ -1510,7 +1637,7 @@ int janus_rtcp_transport_wide_cc_feedback(char *packet, size_t size, guint32 ssr
 		/* Store las status */
 		last_status = status;
 
-		/* Check if we can still be enquing for a run */
+		/* Check if we can still be enqueuing for a run */
 		if (!all_same) {
 			/* Check  */
 			if (!all_same && max_status==janus_rtp_packet_status_largeornegativedelta && g_queue_get_length(statuses)>6) {
@@ -1518,9 +1645,9 @@ int janus_rtcp_transport_wide_cc_feedback(char *packet, size_t size, guint32 ssr
 				/*
 					0                   1
 					0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5
-				       +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
-				       |T|S|        Symbols            |
-				       +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
+					+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
+					|T|S|        Symbols            |
+					+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 					T = 1
 					S = 1
 				 */
@@ -1564,11 +1691,11 @@ int janus_rtcp_transport_wide_cc_feedback(char *packet, size_t size, guint32 ssr
 				/*
 					0                   1
 					0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5
-				       +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
-				       |T|S|       symbol list         |
-				       +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
-					 T = 1
-					 S = 0
+					+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
+					|T|S|       symbol list         |
+					+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
+					T = 1
+					S = 0
 				 */
 				word = janus_push_bits(word, 1, 1);
 				word = janus_push_bits(word, 1, 0);
@@ -1655,9 +1782,15 @@ int janus_rtcp_transport_wide_cc_feedback(char *packet, size_t size, guint32 ssr
 		/* Get next delta */
 		gint delta = GPOINTER_TO_INT(g_queue_pop_head (deltas));
 		/* Check size */
-		if (delta<0 || delta>127) {
+		if (delta<0 || delta>255) {
+			short reported_delta = (short)delta;
+			/* Overflow */
+			if (reported_delta != delta) {
+				reported_delta = delta > 0 ? SHRT_MAX : SHRT_MIN;
+				JANUS_LOG(LOG_ERR, "Delta value (%d) too large, reporting it as %d\n", delta, reported_delta);
+			}
 			/* 2 bytes */
-			janus_set2(data, len, (short)delta);
+			janus_set2(data, len, reported_delta);
 			/* Inc */
 			len += 2;
 		} else {
