@@ -259,12 +259,17 @@ janus_sdp *janus_sdp_parse(const char *sdp, char *error, size_t errlen) {
 	gboolean success = TRUE;
 	janus_sdp_mline *mline = NULL;
 
-	gchar **parts = g_strsplit(sdp, strstr(sdp, "\r\n") ? "\r\n" : "\n", -1);
+	gchar **parts = g_strsplit(sdp, "\n", -1);
 	if(parts) {
 		int index = 0;
-		char *line = NULL;
+		char *line = NULL, *cr = NULL;
 		while(success && (line = parts[index]) != NULL) {
+			cr = strchr(line, '\r');
+			if(cr != NULL)
+				*cr = '\0';
 			if(*line == '\0') {
+				if(cr != NULL)
+					*cr = '\r';
 				index++;
 				continue;
 			}
@@ -446,7 +451,11 @@ janus_sdp *janus_sdp_parse(const char *sdp, char *error, size_t errlen) {
 								m->fmts = g_list_append(m->fmts, g_strdup(mline_parts[mindex]));
 								/* Add numeric payload type */
 								int ptype = atoi(mline_parts[mindex]);
-								m->ptypes = g_list_append(m->ptypes, GINT_TO_POINTER(ptype));
+								if(ptype < 0) {
+									JANUS_LOG(LOG_ERR, "Invalid payload type (%s)\n", mline_parts[mindex]);
+								} else {
+									m->ptypes = g_list_append(m->ptypes, GINT_TO_POINTER(ptype));
+								}
 								mindex++;
 							}
 							g_strfreev(mline_parts);
@@ -514,8 +523,8 @@ janus_sdp *janus_sdp_parse(const char *sdp, char *error, size_t errlen) {
 							break;
 						}
 						*semicolon = '\0';
-						if(strcmp(line, "AS")) {
-							/* We only support b=AS, skip */
+						if(strcmp(line, "AS") && strcmp(line, "TIAS")) {
+							/* We only support b=AS and b=TIAS, skip */
 							break;
 						}
 						mline->b_name = g_strdup(line);
@@ -571,8 +580,12 @@ janus_sdp *janus_sdp_parse(const char *sdp, char *error, size_t errlen) {
 						break;
 				}
 			}
+			if(cr != NULL)
+				*cr = '\r';
 			index++;
 		}
+		if(cr != NULL)
+			*cr = '\r';
 		g_strfreev(parts);
 	}
 	/* FIXME Do a last check: is all the stuff that's supposed to be there available? */
@@ -676,8 +689,11 @@ int janus_sdp_get_codec_pt(janus_sdp *sdp, const char *codec) {
 			janus_sdp_attribute *a = (janus_sdp_attribute *)ma->data;
 			if(a->name != NULL && a->value != NULL && !strcasecmp(a->name, "rtpmap")) {
 				int pt = atoi(a->value);
-				if(strstr(a->value, format) || strstr(a->value, format2))
+				if(pt < 0) {
+					JANUS_LOG(LOG_ERR, "Invalid payload type (%s)\n", a->value);
+				} else if(strstr(a->value, format) || strstr(a->value, format2)) {
 					return pt;
+				}
 			}
 			ma = ma->next;
 		}
@@ -995,6 +1011,9 @@ janus_sdp *janus_sdp_new(const char *name, const char *address) {
 	return sdp;
 }
 
+static int janus_sdp_id_compare(gconstpointer a, gconstpointer b) {
+	return GPOINTER_TO_INT(a) - GPOINTER_TO_INT(b);
+}
 janus_sdp *janus_sdp_generate_offer(const char *name, const char *address, ...) {
 	/* This method has a variable list of arguments, telling us what we should offer */
 	va_list args;
@@ -1006,6 +1025,8 @@ janus_sdp *janus_sdp_generate_offer(const char *name, const char *address, ...)
 	const char *audio_codec = NULL, *video_codec = NULL, *audio_fmtp = NULL;
 	int audio_pt = 111, video_pt = 96;
 	janus_sdp_mdirection audio_dir = JANUS_SDP_SENDRECV, video_dir = JANUS_SDP_SENDRECV;
+	GHashTable *audio_extmaps = NULL, *audio_extids = NULL,
+		*video_extmaps = NULL, *video_extids = NULL;
 	int property = va_arg(args, int);
 	while(property != JANUS_SDP_OA_DONE) {
 		if(property == JANUS_SDP_OA_AUDIO) {
@@ -1036,6 +1057,45 @@ janus_sdp *janus_sdp_generate_offer(const char *name, const char *address, ...)
 			h264_fmtp = va_arg(args, gboolean);
 		} else if(property == JANUS_SDP_OA_DATA_LEGACY) {
 			data_legacy = va_arg(args, gboolean);
+		} else if(property == JANUS_SDP_OA_AUDIO_EXTENSION || property == JANUS_SDP_OA_VIDEO_EXTENSION) {
+			char *extmap = va_arg(args, char *);
+			int id = va_arg(args, int);
+			if(extmap != NULL && id > 0 && id < 15) {
+				if(audio_extmaps == NULL)
+					audio_extmaps = g_hash_table_new(g_str_hash, g_str_equal);
+				if(audio_extids == NULL)
+					audio_extids = g_hash_table_new(NULL, NULL);
+				if(video_extmaps == NULL)
+					video_extmaps = g_hash_table_new(g_str_hash, g_str_equal);
+				if(video_extids == NULL)
+					video_extids = g_hash_table_new(NULL, NULL);
+				/* Make sure the extmap and ID have not been added already */
+				char *audio_extmap = g_hash_table_lookup(audio_extids, GINT_TO_POINTER(id));
+				char *video_extmap = g_hash_table_lookup(video_extids, GINT_TO_POINTER(id));
+				if((property == JANUS_SDP_OA_VIDEO_EXTENSION && audio_extmap != NULL && strcasecmp(audio_extmap, extmap)) ||
+						(property == JANUS_SDP_OA_AUDIO_EXTENSION && video_extmap != NULL && strcasecmp(video_extmap, extmap))) {
+					JANUS_LOG(LOG_WARN, "Ignoring duplicate extension %d (already added: %s)\n",
+						id, audio_extmap ? audio_extmap : video_extmap);
+				} else {
+					if(property == JANUS_SDP_OA_AUDIO_EXTENSION) {
+						if(g_hash_table_lookup(audio_extmaps, extmap) != NULL) {
+							JANUS_LOG(LOG_WARN, "Ignoring duplicate audio extension %s (already added: %d)\n",
+								extmap, GPOINTER_TO_INT(g_hash_table_lookup(audio_extmaps, extmap)));
+						} else {
+							g_hash_table_insert(audio_extmaps, extmap, GINT_TO_POINTER(id));
+							g_hash_table_insert(audio_extids, GINT_TO_POINTER(id), extmap);
+						}
+					} else {
+						if(g_hash_table_lookup(video_extmaps, extmap) != NULL) {
+							JANUS_LOG(LOG_WARN, "Ignoring duplicate video extension %s (already added: %d)\n",
+								extmap, GPOINTER_TO_INT(g_hash_table_lookup(video_extmaps, extmap)));
+						} else {
+							g_hash_table_insert(video_extmaps, extmap, GINT_TO_POINTER(id));
+							g_hash_table_insert(video_extids, GINT_TO_POINTER(id), extmap);
+						}
+					}
+				}
+			}
 		} else {
 			JANUS_LOG(LOG_WARN, "Unknown property %d for preparing SDP answer, ignoring...\n", property);
 		}
@@ -1047,6 +1107,14 @@ janus_sdp *janus_sdp_generate_offer(const char *name, const char *address, ...)
 	if(do_audio && audio_rtpmap == NULL) {
 		JANUS_LOG(LOG_ERR, "Unsupported audio codec '%s', can't prepare an offer\n", audio_codec);
 		va_end(args);
+		if(audio_extmaps != NULL)
+			g_hash_table_destroy(audio_extmaps);
+		if(audio_extids != NULL)
+			g_hash_table_destroy(audio_extids);
+		if(video_extmaps != NULL)
+			g_hash_table_destroy(video_extmaps);
+		if(video_extids != NULL)
+			g_hash_table_destroy(video_extids);
 		return NULL;
 	}
 	if(video_codec == NULL)
@@ -1055,6 +1123,14 @@ janus_sdp *janus_sdp_generate_offer(const char *name, const char *address, ...)
 	if(do_video && video_rtpmap == NULL) {
 		JANUS_LOG(LOG_ERR, "Unsupported video codec '%s', can't prepare an offer\n", video_codec);
 		va_end(args);
+		if(audio_extmaps != NULL)
+			g_hash_table_destroy(audio_extmaps);
+		if(audio_extids != NULL)
+			g_hash_table_destroy(audio_extids);
+		if(video_extmaps != NULL)
+			g_hash_table_destroy(video_extmaps);
+		if(video_extids != NULL)
+			g_hash_table_destroy(video_extids);
 		return NULL;
 	}
 #ifndef HAVE_SCTP
@@ -1085,6 +1161,23 @@ janus_sdp *janus_sdp_generate_offer(const char *name, const char *address, ...)
 			janus_sdp_attribute *a = janus_sdp_attribute_create("fmtp", "%d %s", audio_pt, audio_fmtp);
 			m->attributes = g_list_append(m->attributes, a);
 		}
+		/* Check if we need to add audio extensions to the SDP */
+		if(audio_extids != NULL) {
+			GList *ids = g_list_sort(g_hash_table_get_keys(audio_extids), janus_sdp_id_compare), *iter = ids;
+			while(iter) {
+				char *extmap = g_hash_table_lookup(audio_extids, iter->data);
+				if(extmap != NULL) {
+					janus_sdp_attribute *a = janus_sdp_attribute_create("extmap",
+						"%d %s\r\n", GPOINTER_TO_INT(iter->data), extmap);
+					janus_sdp_attribute_add_to_mline(m, a);
+				}
+				iter = iter->next;
+			}
+			g_list_free(ids);
+		}
+		/* It is safe to add transport-wide rtcp feedback message here, won't be used unless the header extension is negotiated */
+		a = janus_sdp_attribute_create("rtcp-fb", "%d transport-cc", audio_pt);
+		m->attributes = g_list_append(m->attributes, a);
 		offer->m_lines = g_list_append(offer->m_lines, m);
 	}
 	if(do_video) {
@@ -1111,6 +1204,20 @@ janus_sdp *janus_sdp_generate_offer(const char *name, const char *address, ...)
 			a = janus_sdp_attribute_create("rtcp-fb", "%d goog-remb", video_pt);
 			m->attributes = g_list_append(m->attributes, a);
 		}
+		/* Check if we need to add audio extensions to the SDP */
+		if(video_extids != NULL) {
+			GList *ids = g_list_sort(g_hash_table_get_keys(video_extids), janus_sdp_id_compare), *iter = ids;
+			while(iter) {
+				char *extmap = g_hash_table_lookup(video_extids, iter->data);
+				if(extmap != NULL) {
+					janus_sdp_attribute *a = janus_sdp_attribute_create("extmap",
+						"%d %s\r\n", GPOINTER_TO_INT(iter->data), extmap);
+					janus_sdp_attribute_add_to_mline(m, a);
+				}
+				iter = iter->next;
+			}
+			g_list_free(ids);
+		}
 		offer->m_lines = g_list_append(offer->m_lines, m);
 	}
 	if(do_data) {
@@ -1129,6 +1236,14 @@ janus_sdp *janus_sdp_generate_offer(const char *name, const char *address, ...)
 		}
 		offer->m_lines = g_list_append(offer->m_lines, m);
 	}
+	if(audio_extmaps != NULL)
+		g_hash_table_destroy(audio_extmaps);
+	if(audio_extids != NULL)
+		g_hash_table_destroy(audio_extids);
+	if(video_extmaps != NULL)
+		g_hash_table_destroy(video_extmaps);
+	if(video_extids != NULL)
+		g_hash_table_destroy(video_extids);
 
 	/* Done */
 	va_end(args);
@@ -1342,6 +1457,7 @@ janus_sdp *janus_sdp_generate_answer(janus_sdp *offer, ...) {
 				/* Reject */
 				JANUS_LOG(LOG_WARN, "Couldn't find codec we needed (%s) in the offer, rejecting %s\n",
 					codec, m->type == JANUS_SDP_AUDIO ? "audio" : "video");
+				am->port = 0;
 				am->direction = JANUS_SDP_INACTIVE;
 				temp = temp->next;
 				continue;
@@ -1350,43 +1466,49 @@ janus_sdp *janus_sdp_generate_answer(janus_sdp *offer, ...) {
 			/* Add the related attributes */
 			if(m->type == JANUS_SDP_AUDIO) {
 				/* Add rtpmap attribute */
-				janus_sdp_attribute *a = janus_sdp_attribute_create("rtpmap", "%d %s", pt, janus_sdp_get_codec_rtpmap(codec));
-				am->attributes = g_list_append(am->attributes, a);
-				/* Check if we need to add a payload type for DTMF tones (telephone-event/8000) */
-				if(audio_dtmf) {
-					int dtmf_pt = janus_sdp_get_codec_pt(offer, "dtmf");
-					if(dtmf_pt >= 0) {
-						/* We do */
-						am->ptypes = g_list_append(am->ptypes, GINT_TO_POINTER(dtmf_pt));
-						janus_sdp_attribute *a = janus_sdp_attribute_create("rtpmap", "%d %s", dtmf_pt, janus_sdp_get_codec_rtpmap("dtmf"));
+				const char *codec_rtpmap = janus_sdp_get_codec_rtpmap(codec);
+				if(codec_rtpmap) {
+					janus_sdp_attribute *a = janus_sdp_attribute_create("rtpmap", "%d %s", pt, codec_rtpmap);
+					am->attributes = g_list_append(am->attributes, a);
+					/* Check if we need to add a payload type for DTMF tones (telephone-event/8000) */
+					if(audio_dtmf) {
+						int dtmf_pt = janus_sdp_get_codec_pt(offer, "dtmf");
+						if(dtmf_pt >= 0) {
+							/* We do */
+							am->ptypes = g_list_append(am->ptypes, GINT_TO_POINTER(dtmf_pt));
+							janus_sdp_attribute *a = janus_sdp_attribute_create("rtpmap", "%d %s", dtmf_pt, janus_sdp_get_codec_rtpmap("dtmf"));
+							am->attributes = g_list_append(am->attributes, a);
+						}
+					}
+					/* Check if there's a custom fmtp line to add for audio
+					 * FIXME We should actually check if it matches the offer */
+					if(audio_fmtp) {
+						janus_sdp_attribute *a = janus_sdp_attribute_create("fmtp", "%d %s", pt, audio_fmtp);
 						am->attributes = g_list_append(am->attributes, a);
 					}
 				}
-				/* Check if there's a custom fmtp line to add for audio
-				 * FIXME We should actually check if it matches the offer */
-				if(audio_fmtp) {
-					janus_sdp_attribute *a = janus_sdp_attribute_create("fmtp", "%d %s", pt, audio_fmtp);
-					am->attributes = g_list_append(am->attributes, a);
-				}
 			} else {
 				/* Add rtpmap attribute */
-				janus_sdp_attribute *a = janus_sdp_attribute_create("rtpmap", "%d %s", pt, janus_sdp_get_codec_rtpmap(codec));
-				am->attributes = g_list_append(am->attributes, a);
-				if(!strcasecmp(codec, "h264") && h264_fmtp) {
-					/* If it's H.264 and we were asked to, add the default fmtp profile as well */
-					a = janus_sdp_attribute_create("fmtp", "%d profile-level-id=42e01f;packetization-mode=1", pt);
-					am->attributes = g_list_append(am->attributes, a);
-				}
-				if(video_rtcpfb) {
-					/* Add rtcp-fb attributes */
-					a = janus_sdp_attribute_create("rtcp-fb", "%d ccm fir", pt);
-					am->attributes = g_list_append(am->attributes, a);
-					a = janus_sdp_attribute_create("rtcp-fb", "%d nack", pt);
-					am->attributes = g_list_append(am->attributes, a);
-					a = janus_sdp_attribute_create("rtcp-fb", "%d nack pli", pt);
-					am->attributes = g_list_append(am->attributes, a);
-					a = janus_sdp_attribute_create("rtcp-fb", "%d goog-remb", pt);
+				const char *codec_rtpmap = janus_sdp_get_codec_rtpmap(codec);
+				if(codec_rtpmap) {
+					janus_sdp_attribute *a = janus_sdp_attribute_create("rtpmap", "%d %s", pt, codec_rtpmap);
 					am->attributes = g_list_append(am->attributes, a);
+					if(!strcasecmp(codec, "h264") && h264_fmtp) {
+						/* If it's H.264 and we were asked to, add the default fmtp profile as well */
+						a = janus_sdp_attribute_create("fmtp", "%d profile-level-id=42e01f;packetization-mode=1", pt);
+						am->attributes = g_list_append(am->attributes, a);
+					}
+					if(video_rtcpfb) {
+						/* Add rtcp-fb attributes */
+						a = janus_sdp_attribute_create("rtcp-fb", "%d ccm fir", pt);
+						am->attributes = g_list_append(am->attributes, a);
+						a = janus_sdp_attribute_create("rtcp-fb", "%d nack", pt);
+						am->attributes = g_list_append(am->attributes, a);
+						a = janus_sdp_attribute_create("rtcp-fb", "%d nack pli", pt);
+						am->attributes = g_list_append(am->attributes, a);
+						a = janus_sdp_attribute_create("rtcp-fb", "%d goog-remb", pt);
+						am->attributes = g_list_append(am->attributes, a);
+					}
 					/* It is safe to add transport-wide rtcp feedback mesage here, won't be used unless the header extension is negotiated*/
 					a = janus_sdp_attribute_create("rtcp-fb", "%d transport-cc", pt);
 					am->attributes = g_list_append(am->attributes, a);
@@ -1405,6 +1527,11 @@ janus_sdp *janus_sdp_generate_answer(janus_sdp *offer, ...) {
 							if(strstr(a->value, extension)) {
 								/* Accept the extension */
 								int id = atoi(a->value);
+								if(id < 0) {
+									JANUS_LOG(LOG_ERR, "Invalid extension ID (%d)\n", id);
+									temp = temp->next;
+									continue;
+								}
 								const char *direction = NULL;
 								switch(a->direction) {
 									case JANUS_SDP_SENDONLY:
