@@ -12,8 +12,8 @@
  * \ref core
  */
 
-#ifndef _JANUS_SDP_UTILS_H
-#define _JANUS_SDP_UTILS_H
+#ifndef JANUS_SDP_UTILS_H
+#define JANUS_SDP_UTILS_H
 
 
 #include <inttypes.h>
@@ -274,6 +274,10 @@ JANUS_SDP_OA_VIDEO_RTCPFB_DEFAULTS,
 JANUS_SDP_OA_VIDEO_H264_FMTP,
 /*! \brief When generating an offer (this is ignored for answers), use the old "DTLS/SCTP" instead of the new "UDP/DTLS/SCTP (default=TRUE for now, depends on what follows) */
 JANUS_SDP_OA_DATA_LEGACY,
+/*! \brief When generating an offer (this is ignored for answers), negotiate this audio extension: needs two arguments, extmap value and extension ID; can be used multiple times) */
+JANUS_SDP_OA_AUDIO_EXTENSION,
+/*! \brief When generating an offer (this is ignored for answers), negotiate this video extension: needs two arguments, extmap value and extension ID; can be used multiple times) */
+JANUS_SDP_OA_VIDEO_EXTENSION,
 /*! \brief When generating an answer (this is ignored for offers), accept this extension (by default, we reject them all; can be used multiple times) */
 JANUS_SDP_OA_ACCEPT_EXTMAP,
 /*! \brief MUST be used as the last argument in janus_sdp_generate_offer and janus_sdp_generate_answer */
